package org.processmining.processtree;

import java.util.Collection;

import org.processmining.plugins.properties.processmodel.ManualTask;

public interface Task extends Node {

	/**
	 * Automatic task
	 * 
	 * @author bfvdonge && dschunse
	 * 
	 */
	public static interface Automatic extends Task {

	}

	/**
	 * Manual task
	 * 
	 * @author bfvdonge
	 * 
	 */
	public static interface Manual extends Task, ManualTask {

		/**
		 * Returns a non-empty collection of originators attached to this manual
		 * task
		 * 
		 * @return
		 */
		public Collection<Originator> getOriginators();
		
		
		/**
		 * Returns a collection of originators removable from this manual
		 * task
		 * @return
		 */
		public Collection<Originator> getRemovableOriginators();
		
		/**
		 * Adds a single originator to this task
		 * @param originator
		 * @return
		 */
		public boolean addOriginator(Originator originator);
		
		
		/**
		 * Adds a single removable originator to this task
		 * @param removableOriginator
		 * @return
		 */
		public boolean addRemovableOriginator(Originator removableOriginator);

		/**
		 * Removes an originator from the set of originators
		 * @param originator
		 * @return
		 */
		public boolean removeOriginator(Originator originator);
		
		/**
		 * Set the collection of originators attached to this manual task
		 */
		public void addOriginators(Collection<Originator> originators);
		
		/**
		 * Set the collection of removable originators attached to this manual task
		 */
		public void addRemovableOriginators(Collection<Originator> removableOriginators);
	}
}
