package org.processmining.processtree.configuration.controlflow.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;
import java.util.UUID;

import org.processmining.processtree.Block.PlaceHolder;
import org.processmining.processtree.Edge;
import org.processmining.processtree.Node;
import org.processmining.processtree.ProcessTree;
import org.processmining.processtree.ProcessTree.Type;
import org.processmining.processtree.configuration.controlflow.ControlFlowConfiguration;

public class ControlFlowConfigurationImpl implements ControlFlowConfiguration {
	Set<UUID> blockedEdges;
	Set<UUID> nonBlockedEdges;
	Set<UUID> hiddenEdges;
	Set<UUID> nonHiddenEdges;
	Map<UUID, UUID> selectedReplacements;
	Map<UUID, List<UUID>> nonSelectedReplacements;
	Map<UUID, Type> replacedNodes;
	Set<UUID> nonReplacedNodes;

	public ControlFlowConfigurationImpl() {
		blockedEdges = new HashSet<UUID>();
		nonBlockedEdges = new HashSet<UUID>();
		hiddenEdges = new HashSet<UUID>();
		nonHiddenEdges = new HashSet<UUID>();
		selectedReplacements = new HashMap<UUID, UUID>();
		nonSelectedReplacements = new HashMap<UUID, List<UUID>>();
		replacedNodes = new HashMap<UUID, Type>();
		nonReplacedNodes = new HashSet<UUID>();
	}

	public void addBlockedEdge(Edge e) {
		blockedEdges.add(e.getID());

	}

	public void addNonBlockedEdge(Edge e) {
		nonBlockedEdges.add(e.getID());

	}

	public void addHiddenEdge(Edge e) {
		hiddenEdges.add(e.getID());

	}

	public void addNonHiddenEdge(Edge e) {
		nonHiddenEdges.add(e.getID());

	}

	public void setSelectedReplacement(PlaceHolder placeholder, Node selectedChild) {
		selectedReplacements.put(placeholder.getID(), selectedChild.getID());
	}

	public void addNonSelectedReplacement(PlaceHolder placeholder, Node nonSelectedChild) {
		if (nonSelectedReplacements.get(placeholder.getID()) == null) {
			nonSelectedReplacements.put(placeholder.getID(), new ArrayList<UUID>());
		}
		nonSelectedReplacements.get(placeholder.getID()).add(nonSelectedChild.getID());
	}

	public void setChangeableNode(Node node, Type type) {
		replacedNodes.put(node.getID(), type);
	}

	public void addNonChangeableNode(Node node) {
		nonReplacedNodes.add(node.getID());

	}

	public boolean isBlocked(Edge e) {
		return blockedEdges.contains(e.getID());
	}

	public boolean isNonBlocked(Edge e) {
		return nonBlockedEdges.contains(e.getID());
	}

	public boolean isHidden(Edge e) {
		return hiddenEdges.contains(e.getID());
	}

	public boolean isNonHidden(Edge e) {
		return nonHiddenEdges.contains(e.getID());
	}

	public Set<UUID> getBlockedEdges() {
		return blockedEdges;
	}

	public Set<UUID> getNonBlockedEdges() {
		return nonBlockedEdges;
	}

	public Set<UUID> getHiddenEdges() {
		return hiddenEdges;
	}

	public Set<UUID> getNonHiddenEdges() {
		return nonHiddenEdges;
	}

	public Map<UUID, UUID> getSelectedReplacements() {
		return selectedReplacements;
	}

	public Map<UUID, List<UUID>> getNonSelectedReplacements() {
		return nonSelectedReplacements;
	}

	public Map<UUID, Type> getReplacedNodes() {
		return replacedNodes;
	}

	public Set<UUID> getNonReplacedNodes() {
		return nonReplacedNodes;
	}

	public void addBlockedEdge(UUID e) {
		blockedEdges.add(e);
	}

	public void addNonBlockedEdge(UUID e) {
		nonBlockedEdges.add(e);
	}

	public void addHiddenEdge(UUID e) {
		hiddenEdges.add(e);
	}

	public void addNonHiddenEdge(UUID e) {
		nonHiddenEdges.add(e);
	}

	public void setSelectedReplacement(UUID placeholder, UUID selectedChild) {
		selectedReplacements.put(placeholder, selectedChild);
	}

	public void addNonSelectedReplacement(UUID placeholder, UUID nonSelectedChild) {
		if (nonSelectedReplacements.get(placeholder) == null) {
			nonSelectedReplacements.put(placeholder, new ArrayList<UUID>());
		}
		nonSelectedReplacements.get(placeholder).add(nonSelectedChild);
	}

	public void setChangeableNode(UUID node, Type type) {
		replacedNodes.put(node, type);
	}

	public void addNonChangeableNode(UUID node) {
		nonReplacedNodes.add(node);
	}

	public String toString() {
		return toString(null);
	}

	/**
	 * Special version of the toString method where the UUIDs are searched for
	 * in the provided process tree (NULL allowed) and the appropriate toString
	 * method is called for a more intuitive string result
	 * 
	 * @param pt
	 * @return
	 */
	public String toString(ProcessTree pt) {
		StringBuilder str = new StringBuilder();

		str.append("[ blockedEdges: {");
		for (UUID id : blockedEdges) {
			str.append((pt == null ? id : pt.getEdge(id).toString()) + " ");
		}

		str.append("} nonBlockedEdges: {");
		for (UUID id : nonBlockedEdges) {
			str.append((pt == null ? id : pt.getEdge(id).toString()) + " ");
		}

		str.append("} hiddenEdges: {");
		for (UUID id : hiddenEdges) {
			str.append((pt == null ? id : pt.getEdge(id).toString()) + " ");
		}

		str.append("} nonHiddenEdges: {");
		for (UUID id : nonHiddenEdges) {
			str.append((pt == null ? id : pt.getEdge(id).toString()) + " ");
		}

		str.append("} replacedNodes: {");
		for (Entry<UUID, Type> entry : replacedNodes.entrySet()) {
			str.append((pt == null ? entry.toString() : pt.getNode(entry.getKey()) + "->" + entry.getValue().toString())
					+ " ");
		}

		str.append("} nonReplacedNodes: {");
		for (UUID id : nonReplacedNodes) {
			str.append((pt == null ? id : pt.getNode(id).toString()) + " ");
		}

		str.append("} selectedReplacements: {");
		for (Entry<UUID, UUID> entry : selectedReplacements.entrySet()) {
			str.append((pt == null ? entry.toString() : pt.getNode(entry.getKey()) + "=>"
					+ pt.getNode(entry.getValue()))
					+ " ");
		}

		str.append("} nonSelectedReplacements: {");
		for (Entry<UUID, List<UUID>> entry : nonSelectedReplacements.entrySet()) {
			str.append((pt == null ? entry.toString() : pt.getNode(entry.getKey()) + "=>{"));

			for (UUID id : entry.getValue()) {
				str.append((pt == null ? id : pt.getNode(id).toString()) + " ");
			}

			str.append("} ");

			//str.append(entry.toString() + " ");
		}
		str.append("}]");

		return str.toString();
	}
	
	public void addBlockedEdges(Set<UUID> ids) {
		blockedEdges.addAll(ids);
	}

	public void addNonBlockedEdges(Set<UUID> ids) {
		nonBlockedEdges.addAll(ids);
	}

	public void addHiddenEdges(Set<UUID> ids) {
		hiddenEdges.addAll(ids);
	}

	public void addNonHiddenEdges(Set<UUID> ids) {
		nonHiddenEdges.addAll(ids);
	}
}
