package org.processmining.processtree.configuration.controlflow.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.Stack;
import java.util.UUID;

import org.processmining.processtree.Block;
import org.processmining.processtree.Block.DefLoop;
import org.processmining.processtree.Block.PlaceHolder;
import org.processmining.processtree.Block.Seq;
import org.processmining.processtree.Block.XorLoop;
import org.processmining.processtree.Edge;
import org.processmining.processtree.Event;
import org.processmining.processtree.Node;
import org.processmining.processtree.ProcessTree;
import org.processmining.processtree.Task.Automatic;
import org.processmining.processtree.configuration.ConfigurationAuxiliary;
import org.processmining.processtree.configuration.controlflow.ControlFlowConfiguration;
import org.processmining.processtree.configuration.controlflow.ControlFlowConfigurationInstantiater;
import org.processmining.processtree.impl.AbstractBlock;
import org.processmining.processtree.impl.AbstractTask;
import org.processmining.processtree.impl.ProcessTreeImpl;

public class ControlFlowConfigurationInstantiaterImpl implements ControlFlowConfigurationInstantiater{
	
	/**
	 * CURRENT ISSUES:
	 *  - Replacement option which is blocked (directly on the edge)
	 *  - Replacement option which is hidden (directly on the edge)
	 *  - No support for operator downgrading
	 *  - Check for parent with twins, or triplets, or...
	 */
	
	private Map<UUID, Set<UUID>> hidingId;
	
	public ProcessTree instantiate(ProcessTree treeIn, ControlFlowConfiguration conf) {
		return instantiate(treeIn, conf, false);
	}
	
	public ProcessTree instantiate(ProcessTree treeIn, ControlFlowConfiguration conf, Boolean makeConfigurationComplete){
		hidingId = new HashMap<UUID, Set<UUID>>();
		if(makeConfigurationComplete){
			ControlFlowConfigurationIteratorImpl.makeComplete(conf, treeIn);
		}
		ProcessTree treeOut = new ProcessTreeImpl(treeIn);
		treeOut = applySubstitution(treeOut, conf);
		treeOut = applyHiding(treeOut, conf, new AbstractTask.Automatic("tau"));
		treeOut = applyBlocking(treeOut, conf);
		if(treeOut != null){
			treeOut = removeDisconnectedParts(treeOut);
			return treeOut;
		}
		else{
			// when treeOut is null, we return an empty tree (de excuus truus :P)
			//treeOut = new ProcessTreeImpl();
			//Automatic tau = new AbstractTask.Automatic("Excuus Truus");
			//tau.setProcessTree(treeOut);
			//treeOut.addNode(tau);
			//treeOut.setRoot(tau);
			//return treeOut;
			return null;
		}
	}
	
	public Map<UUID, Set<UUID>> getMappingHiddenNodesToTaus(){
		return hidingId;
	}

	private ProcessTree applySubstitution(ProcessTree tree, ControlFlowConfiguration conf){
		// we have to substitute the placeholders with their selected children
		for(UUID n: conf.getSelectedReplacements().keySet()){
			PlaceHolder treeNodePlaceholder = (PlaceHolder) tree.getNode(n);
			Node treeNodeTarget = tree.getNode(conf.getSelectedReplacements().get(n));
			if(treeNodeTarget == null){
				// oeps we have a problem
				// TODO: can happen when we have a placeholder node with a placeholder node as child which has already been selected
				assert false;
			}
			// we have the placeholder node, get her parent(s)
			List<Edge> inComingEdges = treeNodePlaceholder.getIncomingEdges();
			for(Edge e: inComingEdges){				
				e.setTarget(treeNodeTarget);
				treeNodeTarget.addIncomingEdge(e);
			}
			if(tree.getRoot().equals(treeNodePlaceholder)){
				// we have to update the root
				tree.setRoot(treeNodeTarget);
			}
			tree.removeNode(treeNodePlaceholder);
			for(Edge e: treeNodePlaceholder.getOutgoingEdges()){
				e.getTarget().removeIncomingEdge(e);
				tree.removeEdge(e);
			}
			// we also have to update the configuration because there might be a placeholder pointing to this removed placeholder node
			for(UUID p: conf.getSelectedReplacements().keySet()){
				if(conf.getSelectedReplacements().get(p).equals(treeNodePlaceholder)){
					// now update the configuration
					conf.setSelectedReplacement(p, treeNodeTarget.getID());
				}
			}
		}
		for(UUID n: conf.getNonSelectedReplacements().keySet()){
			PlaceHolder treeNodePlaceholder = (PlaceHolder) tree.getNode(n);
			if(treeNodePlaceholder == null){
				// don't bother we already removed it
				break;
			}
			for(UUID child: conf.getNonSelectedReplacements().get(n)){
				Node treeNodeTarget = tree.getNode(child);
				Edge eRem = null;
				for(Edge e: treeNodePlaceholder.getOutgoingEdges()){
					if(e.getTarget().equals(treeNodeTarget)){
						eRem = e;
						break;
					}
				}
				treeNodePlaceholder.removeOutgoingEdge(eRem);
				treeNodeTarget.removeIncomingEdge(eRem);
			}
		}
		return tree;
	}
	
	private ProcessTree applyHiding(ProcessTree tree, ControlFlowConfiguration conf, Automatic tau){
		// we have a automatic task in case we want a tau task for hiding parts (we need to copy it in case we want to maintain a tree)
		for(UUID e: conf.getHiddenEdges()){
			Automatic taup = new AbstractTask.Automatic(UUID.randomUUID(), tau);
			tree.addNode(taup);
			taup.setProcessTree(tree);
			Edge treeEdge = tree.getEdge(e);
			if(treeEdge != null){
				
				if(hidingId.get(treeEdge.getTarget().getID()) == null){
					hidingId.put(treeEdge.getTarget().getID(), new HashSet<UUID>());
				}
				hidingId.get(treeEdge.getTarget().getID()).add(taup.getID());
				
				tree.getNode(treeEdge.getTarget().getID()).removeIncomingEdge(treeEdge);
				treeEdge.setTarget(taup);
				taup.addIncomingEdge(treeEdge);
				treeEdge.setHideable(false);
			}
		}
		for(UUID e: conf.getNonHiddenEdges()){
			// just update these edges by setting hideable to false
			Edge treeEdge = tree.getEdge(e);
			if(treeEdge != null){
				treeEdge.setHideable(false);
			}
		}
		return tree;
	}
	
	private ProcessTree applyBlocking(ProcessTree tree, ControlFlowConfiguration conf){
		// just remove the edges from the process tree, note that we have to do something special in case of SEQ, AND, and LOOP,
		// we need to have a fixpoint iteration, just store the size
		Set<UUID> blockedEdges = ConfigurationAuxiliary.computeClosureBlocking(tree, conf.getBlockedEdges());
		// now we have a pile of edges which can be removed 
		for(UUID e: blockedEdges){
			Edge treeEdge = tree.getEdge(e);
			if(treeEdge == null){
				// do not bother, already removed
			}
			else{
				tree.removeEdge(treeEdge);
				treeEdge.getSource().removeOutgoingEdge(treeEdge);
				treeEdge.getTarget().removeIncomingEdge(treeEdge);
			}
		}
		// now we have removed all of the edges, now remove the nodes
		Set<Node> nodesToRemove = new HashSet<Node>();
		for(Node n: tree.getNodes()){
			if(!tree.getRoot().equals(n)){
				if(n.getIncomingEdges().size() == 0){
					nodesToRemove.add(n);
				}
			}
			if(n instanceof Block){
				// also check the outgoing edges
				if(((Block)n).getOutgoingEdges().size() == 0){
					nodesToRemove.add(n);
				}
			}
		}
		for(Node n: nodesToRemove){
			tree.removeNode(n);
			if(tree.getRoot() != null && tree.getRoot().equals(n)){
				tree.setRoot(null);
			}
		}
		// now make sequences of LOOPS if the case
		HashSet<Block> substituteNodes = new HashSet<Block>();
		for(Node n: tree.getNodes()){
			if(n instanceof DefLoop || n instanceof XorLoop){
				if(((Block)n).getOutgoingEdges().size() == 2){
					// simple test
					// now substitute this node
					substituteNodes.add((Block)n);
				}
			}
		}
		for(Block n: substituteNodes){
			substituteNodeBySequence(tree, n);
		}
		
		// now remove the configuration options of the nodes not blocked
		for(UUID id: conf.getNonBlockedEdges()){
			Edge treeEdge = tree.getEdge(id);
			if(treeEdge != null){
				treeEdge.setBlockable(false);
			}
		}
		if(tree.getRoot() == null){
			return null;
		}
		return tree;
	}
	
	private void substituteNodeBySequence(ProcessTree tree, Block nodeOld){
		Seq s = new AbstractBlock.Seq(nodeOld.getID(), nodeOld.getName(), new ArrayList<>(nodeOld.getIncomingEdges()), new ArrayList<>(nodeOld.getOutgoingEdges()));
		tree.removeNode(nodeOld);
		tree.addNode(s);
		// now update all the edges
		for(Edge e: s.getIncomingEdges()){
			e.setTarget(s);
		}
		for(Edge e: s.getOutgoingEdges()){
			e.setSource(s);
		}
	}
	
	private ProcessTree removeDisconnectedParts(ProcessTree tree) {
		// lets go over the different elements
		int oldSize = 0;
		int newSize = tree.getElements().size();
		while(oldSize != newSize){
			oldSize = newSize;
			Set<Node> nodesToRemove = new HashSet<Node>();
			Set<Edge> edgesToRemove = new HashSet<Edge>();
			for(Node n: tree.getNodes()){
				if(n.getIncomingEdges().size() == 0 && !tree.getRoot().equals(n)){
					nodesToRemove.add(n);
				}
			}
			for(Node n: nodesToRemove){
				tree.removeNode(n);
				if(n instanceof Block){
					for(Edge e: ((Block)n).getOutgoingEdges()){
						edgesToRemove.add(e);
					}
				}
			}
			for(Edge e: tree.getEdges()){
				if(e.getSource() == null || e.getTarget() == null){
					edgesToRemove.add(e);
				}
				else if(e.getSource() != null && tree.getNode(e.getSource().getID()) == null){
					edgesToRemove.add(e);
				}
				else if(e.getTarget() != null && tree.getNode(e.getTarget().getID()) == null){
					edgesToRemove.add(e);
				}
			}
			for(Edge e: edgesToRemove){
				tree.removeEdge(e);
				if(e.getSource() != null){
					e.getSource().removeOutgoingEdge(e);
				}
				if(e.getTarget() != null){
					if(e.getSource() != null){
						e.getTarget().removeIncomingEdge(e);
					}
					else{
						// I do not know, can we reach here?
						assert false;
					}
				}
			}
			newSize = tree.getElements().size();
		}
		return tree;
	}
	
	@SuppressWarnings("unused")
	private boolean checkIntegrity(ProcessTree tree){
		// now lets do a lot of checks on the process tree
		boolean ret = true;
		// first check the amount of children
		for(Node n: tree.getNodes()){
			if(n instanceof DefLoop){
				ret &= ((DefLoop)n).getOutgoingEdges().size() == 3;
			}
			if(n instanceof XorLoop){
				ret &= ((XorLoop)n).getOutgoingEdges().size() == 3;
			}
			if(n instanceof Event){
				ret &= ((Event)n).getOutgoingEdges().size() == 1;
			}
			if(n instanceof Block){
				for(Edge e: ((Block)n).getOutgoingEdges()){
					ret &= e.getSource() != null && e.getTarget() != null;
				}
			}
		}
		for(Edge e: tree.getEdges()){
			ret &= e.getSource() != null && e.getTarget() != null;
		}
		// also do a recursive check on the tree
		Stack<Node> stack = new Stack<Node>();
		stack.push(tree.getRoot());
		while(!stack.isEmpty()){
			Node n = stack.pop();
			ret &= tree.getNode(n.getID()) != null && n.getProcessTree().equals(tree);
			if(ret == false){
				throw new NullPointerException();
			}
			if(n instanceof Block){
				for(Edge e: ((Block)n).getOutgoingEdges()){
					stack.push(e.getTarget());
					ret &= e.getTarget().getIncomingEdges().contains(e);
					if(ret == false){
						throw new NullPointerException();
					}
				}
			}
		}
		return ret;
	}

}
