package org.processmining.processtree.editor.editorpanel;

import java.awt.event.ActionEvent;
import java.util.HashSet;
import java.util.Set;
import java.util.UUID;

import javax.swing.JSlider;

import org.processmining.processtree.Block;
import org.processmining.processtree.Edge;
import org.processmining.processtree.Node;
import org.processmining.processtree.ProcessTree;
import org.processmining.processtree.editor.graphpanel.GraphPanel;
import org.processmining.processtree.editor.interaction.GraphMouseListener;

public class EditorActionListenerAddEdge implements EditorActionListener{
	ProcessTree tree;
	GraphMouseListener graphMouseListener;
	GraphPanel graphPanel;
	JSlider childIndex;
	
	public EditorActionListenerAddEdge(ProcessTree tree, GraphMouseListener graphMouseListener, GraphPanel graphPanel, JSlider childIndex){
		this.tree = tree;
		this.graphMouseListener = graphMouseListener;
		this.graphPanel = graphPanel;
		this.childIndex = childIndex;
	}

	public void actionPerformed(ActionEvent e) {
		if(isEnabled()){
			UUID parentId = graphMouseListener.getPreviouslySelectedID();
			UUID childId = graphMouseListener.getSelectedID();
			// now just add the edge
			Block parent = (Block) tree.getNode(parentId);
			Node child = tree.getNode(childId);
			Edge edge;
			if(childIndex.getValue() < parent.getChildren().size()){
				edge = parent.addChildAt(child, childIndex.getValue());
			}
			else{
				edge = parent.addChild(child);
			}
			child.addIncomingEdge(edge);
			tree.addEdge(edge);
			
			graphPanel.redraw();
		}	
	}
	
	public boolean isEnabled(){
		UUID parentId = graphMouseListener.getPreviouslySelectedID();
		if(parentId != null){
			if(!(tree.getElement(parentId) instanceof Block)){
				return false;
			}
			UUID childId = graphMouseListener.getSelectedID();
			if(!(tree.getElement(childId) instanceof Node)){
				return false;
			}
			// do an acycly check
			if(!remainsAcyclic(parentId, childId)){
				return false;
			}
			return true;
		}
		return false;
	}
	
	private boolean remainsAcyclic(UUID from, UUID to){
		if(from.equals(to)){
			return false;
		}
		// now do a simple check whether we can reach the from via the to
		Node nodeTo = tree.getNode(to);
		if(nodeTo instanceof Block){
			Set<Node> childrenTransitive = getChildren(nodeTo);
			return !childrenTransitive.contains(tree.getNode(from));
		}
		// it is not a block so it is fine
		return true;
		
	}
	
	private Set<Node> getChildren(Node n){
		Set<Node> children = new HashSet<>();
		if(n instanceof Block){
			for(Node child: ((Block)n).getChildren()){
				children.add(child);
				children.addAll(getChildren(child));
			}
		}
		return children;
	}
}
