package org.processmining.processtree.editor.editorpanel;

import java.awt.event.ActionEvent;
import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

import javax.swing.JComboBox;

import org.processmining.processtree.Block;
import org.processmining.processtree.Block.And;
import org.processmining.processtree.Block.Def;
import org.processmining.processtree.Block.DefLoop;
import org.processmining.processtree.Block.Seq;
import org.processmining.processtree.Block.XorLoop;
import org.processmining.processtree.Edge;
import org.processmining.processtree.Node;
import org.processmining.processtree.Originator;
import org.processmining.processtree.ProcessTree;
import org.processmining.processtree.ProcessTree.Type;
import org.processmining.processtree.editor.graphpanel.GraphPanel;
import org.processmining.processtree.editor.interaction.GraphMouseListener;
import org.processmining.processtree.impl.AbstractBlock;
import org.processmining.processtree.impl.AbstractEvent;
import org.processmining.processtree.impl.AbstractTask;
import org.processmining.processtree.impl.EdgeImpl;

public class EditorActionListenerChangeType implements EditorActionListener{
	ProcessTree tree;
	GraphMouseListener graphMouseListener;
	GraphPanel graphPanel;
	JComboBox<Type> typeBox;
	
	public EditorActionListenerChangeType(ProcessTree tree, GraphMouseListener graphMouseListener, GraphPanel graphPanel, JComboBox<Type> typeBox){
		this.tree = tree;
		this.graphMouseListener = graphMouseListener;
		this.graphPanel = graphPanel;
		this.typeBox = typeBox;
	}
	
	public void actionPerformed(ActionEvent e) {
		if(!isEnabled()){
			return;
		}
		// quick check on the same type
		UUID selectedID = graphMouseListener.getSelectedID();
		Node selectedNode = tree.getNode(selectedID);
		Type type = (Type) typeBox.getSelectedItem();
		if(tree.getType(selectedNode).equals(type)){
			return;
		}
		// create a new block based on the current block and the type
		Node newNode = createNode(selectedNode, type);
		newNode.setProcessTree(tree);
		tree.removeNode(selectedNode);
		tree.addNode(newNode);
		// now update the edges
		for(Edge edge: selectedNode.getIncomingEdges()){
			edge.setTarget(newNode);
		}
		if(selectedNode instanceof Block && newNode instanceof Block){
			for(Edge edge: ((Block)selectedNode).getOutgoingEdges()){
				edge.setSource((Block)newNode);
				// we need to have a special check on expressions
				if(newNode instanceof Seq || newNode instanceof And || newNode instanceof Def || newNode instanceof DefLoop){
					edge.setExpression(EdgeImpl.NOEXPRESSION);
				}
				if(newNode instanceof XorLoop && ((Block)selectedNode).getOutgoingEdges().indexOf(edge) == 0){
					edge.setExpression(EdgeImpl.NOEXPRESSION);
				}
			}
		}
		if(tree.getRoot().equals(selectedNode)){
			tree.setRoot(newNode);
		}
		graphPanel.redraw();
	}

	public boolean isEnabled() {
		UUID selectedID = graphMouseListener.getSelectedID();
		if(selectedID != null && tree.getElement(selectedID) instanceof Node){
			return true;
		}
		return false;
	}
	
	private Node createNode(Node oldNode, Type t){
		Node newNode;
		UUID id = oldNode.getID();
		String name = oldNode.getName();
		List<Edge> incoming = oldNode.getIncomingEdges();
		List<Edge> outgoing = new ArrayList<>();
		if(oldNode instanceof Block){
			outgoing = new ArrayList<>(((Block)oldNode).getOutgoingEdges());
		}
		switch(t){
			case SEQ:
				newNode = new AbstractBlock.Seq(id, name, incoming, outgoing);
				break;
			case AND:
				newNode = new AbstractBlock.And(id, name, incoming, outgoing);
				break;
			case XOR:
				newNode = new AbstractBlock.Xor(id, name, incoming, outgoing);
				break;
			case DEF:
				newNode = new AbstractBlock.Def(id, name, incoming, outgoing);
				break;
			case OR:
				newNode = new AbstractBlock.Or(id, name, incoming, outgoing);
				break;
			case LOOPXOR:
				newNode = new AbstractBlock.XorLoop(id, name, incoming, outgoing);
				break;
			case LOOPDEF:
				newNode = new AbstractBlock.DefLoop(id, name, incoming, outgoing);
				break;
			case PLACEHOLDER:
				newNode = new AbstractBlock.PlaceHolder(id, name, incoming, outgoing);
				break;
			case TIMEOUT:
				newNode = new AbstractEvent.TimeOut(id, name, "", incoming.size() > 0 ? incoming.get(0) : null, outgoing.size() > 0 ? outgoing.get(0) : null);
				break;
			case MESSAGE:
				newNode = new AbstractEvent.Message(id, name, "", incoming.size() > 0 ? incoming.get(0) : null, outgoing.size() > 0 ? outgoing.get(0) : null);
				break;
			case MANTASK:
				newNode = new AbstractTask.Manual(id, name, new ArrayList<Originator>());
				break;
			default:
				newNode = new AbstractTask.Automatic(id, name);
				break;
		}
		return newNode;
	}

}
