package org.processmining.processtree.editor.editorpanel;

import java.awt.Color;
import java.awt.event.ActionEvent;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JComboBox;

import org.processmining.framework.util.Pair;
import org.processmining.processtree.Block;
import org.processmining.processtree.Block.Def;
import org.processmining.processtree.Block.DefLoop;
import org.processmining.processtree.Block.Or;
import org.processmining.processtree.Block.Xor;
import org.processmining.processtree.Block.XorLoop;
import org.processmining.processtree.Edge;
import org.processmining.processtree.Event;
import org.processmining.processtree.Node;
import org.processmining.processtree.ProcessTree;
import org.processmining.processtree.ProcessTreeElement;
import org.processmining.processtree.Task.Automatic;
import org.processmining.processtree.impl.AbstractTask.Manual;

public class EditorActionListenerCheckConsistency implements EditorActionListener{
	ProcessTree tree;
	JButton me;
	JComboBox<String> comboBox;
	
	public EditorActionListenerCheckConsistency(ProcessTree tree, JButton me, JComboBox<String> comboBox){
		this.tree = tree;
		this.me = me;
		this.comboBox = comboBox;
	}
	
	public void actionPerformed(ActionEvent arg0) {
		List<Pair<ProcessTreeElement, String>> wrongElements = isConsistent(tree);
		if(wrongElements.isEmpty()){
			me.setBackground(Color.GREEN);
			comboBox.removeAllItems();
			comboBox.setEnabled(false);
		}
		else{
			me.setBackground(Color.RED);
			comboBox.setEnabled(true);
			comboBox.removeAllItems();
			comboBox.addItem("Inconsistent elements:");
			for(Pair<ProcessTreeElement, String> element: wrongElements){
				comboBox.addItem(element.getFirst().getName() + ": " + element.getSecond());
			}
		}
		
	}

	public boolean isEnabled() {
		return true;
	}
	
	private List<Pair<ProcessTreeElement, String>> isConsistent(ProcessTreeElement element){
		List<Pair<ProcessTreeElement, String>> ret = new ArrayList<>();
		if(element instanceof ProcessTree){
			if(((ProcessTree) element).getRoot() == null){
				ret.add(new Pair<ProcessTreeElement, String>(element, "No root"));
			}
			else{
				ret.addAll(isConsistentNode(((ProcessTree)element).getRoot()));
			}
		}
		return ret;
	}
	
	private List<Pair<ProcessTreeElement, String>> isConsistentNode(Node node){
		List<Pair<ProcessTreeElement, String>> ret = new ArrayList<>();
		if(node instanceof Manual){
			ret.addAll(isConsistentManual((Manual) node));
		}
		if(node instanceof Automatic){
			ret.addAll(isConsistentAutomatic((Automatic) node));
		}
		if(node instanceof Block){
			ret.addAll(isConsistenBlock((Block) node));
		}
		return ret;
	}
	
	private List<Pair<ProcessTreeElement, String>> isConsistentManual(Manual manual){
		List<Pair<ProcessTreeElement, String>> ret = new ArrayList<>();
		if(manual.getOriginators().isEmpty()){
			ret.add(new Pair<ProcessTreeElement, String>(manual, "No originators"));
		}
		return ret;
	}
	
	private List<Pair<ProcessTreeElement, String>> isConsistentAutomatic(Automatic auto){
		return new ArrayList<>();
	}
	
	private List<Pair<ProcessTreeElement, String>> isConsistentEdge(Edge edge){
		List<Pair<ProcessTreeElement, String>> ret = new ArrayList<>();
		ret.addAll(isConsistentNode(edge.getTarget()));
		return ret;
	}
	
	private List<Pair<ProcessTreeElement, String>> isConsistenBlock(Block block){
		List<Pair<ProcessTreeElement, String>> ret = new ArrayList<>();
		if(block.getOutgoingEdges().isEmpty()){
			ret.add(new Pair<ProcessTreeElement, String>(block, "No children"));
		}
		for(Edge e: block.getOutgoingEdges()){
			ret.addAll(isConsistentEdge(e));
		}
		
		if(block instanceof XorLoop || block instanceof DefLoop){
			boolean valid = block.getOutgoingEdges().size() == 3;
			valid &= (block instanceof DefLoop || block.getOutgoingEdges().get(1).hasExpression());
			valid &= (block instanceof DefLoop || block.getOutgoingEdges().get(2).hasExpression());
			valid &= (block instanceof XorLoop || block.getOutgoingEdges().get(1).getTarget() instanceof Event);
			valid &= (block instanceof XorLoop || block.getOutgoingEdges().get(2).getTarget() instanceof Event);
			if(!valid){
				ret.add(new Pair<ProcessTreeElement, String>(block, "# children, expression, or event"));
			}
		}
		if(block instanceof Xor || block instanceof Or){
			for(Edge e: block.getOutgoingEdges()){
				if(!e.hasExpression()){
					ret.add(new Pair<ProcessTreeElement, String>(block, "expression missing on outgoing edge"));
					break;
				}
			}
		}
		if(block instanceof Def){
			for(Edge e: block.getOutgoingEdges()){
				if(!(e.getTarget() instanceof Event)){
					ret.add(new Pair<ProcessTreeElement, String>(block, "event missing as child"));
					break;
				}
			}
		}
		if(block instanceof Event){
			if(block.getOutgoingEdges().size() != 1){
				ret.add(new Pair<ProcessTreeElement, String>(block, "wrong number of children"));
			}
		}
		return ret;
	}

}
