package org.processmining.processtree.editor.editorpanel;

import java.awt.event.ActionEvent;
import java.util.UUID;

import org.processmining.processtree.Block;
import org.processmining.processtree.Edge;
import org.processmining.processtree.Expression;
import org.processmining.processtree.Node;
import org.processmining.processtree.Originator;
import org.processmining.processtree.ProcessTree;
import org.processmining.processtree.ProcessTreeElement;
import org.processmining.processtree.Variable;
import org.processmining.processtree.editor.graphpanel.GraphPanel;
import org.processmining.processtree.editor.interaction.GraphMouseListener;

public class EditorActionListenerDeleteElement implements EditorActionListener{
	ProcessTree tree;
	Class<? extends ProcessTreeElement> elementType1;
	Class<? extends ProcessTreeElement> elementType2;
	GraphMouseListener graphMouseListener;
	GraphPanel graphPanel;
	
	
	public EditorActionListenerDeleteElement(ProcessTree tree, Class<? extends ProcessTreeElement> elementType1, Class<? extends ProcessTreeElement> elementType2, GraphMouseListener graphMouseListener, GraphPanel graphPanel){
		this.tree = tree;
		this.elementType1 = elementType1;
		this.elementType2 = elementType2;
		this.graphMouseListener = graphMouseListener;
		this.graphPanel = graphPanel;
	}
	
	public void actionPerformed(ActionEvent e) {
		if(!isEnabled()){
			return;
		}
		UUID removeID = graphMouseListener.getSelectedID();
		if(removeID != null && tree.getElement(removeID) instanceof Edge){
			removeEdge(tree.getEdge(removeID));
			graphPanel.redraw();
		}
		if(removeID != null && tree.getElement(removeID) instanceof Node){
			removeNode(tree.getNode(removeID));
			graphPanel.redraw();
		}
		if(removeID != null && tree.getElement(removeID) instanceof Originator){
			tree.removeOriginator(tree.getOriginator(removeID));
			graphPanel.redraw();
		}
		if(removeID != null && tree.getElement(removeID) instanceof Variable){
			tree.removeVariable(tree.getVariable(removeID));
			graphPanel.redraw();
		}
		if(removeID != null && tree.getElement(removeID) instanceof Expression){
			tree.removeExpression(tree.getExpression(removeID));
			graphPanel.redraw();
		}
	}
	
	public boolean isEnabled(){
		UUID removeID = graphMouseListener.getSelectedID();
		if(removeID != null && (elementType1.isInstance(tree.getElement(removeID)) || (elementType2 != null && elementType2.isInstance(tree.getElement(removeID))))){
			return true;
		}
		return false;
	}
	
	private void removeEdge(Edge edge){
		removeEdge(edge, null);
	}
	
	private void removeEdge(Edge edge, Node caller){
		if(!edge.getSource().equals(caller)){
			edge.getSource().removeOutgoingEdge(edge);
		}
		if(!edge.getTarget().equals(caller)){
			edge.getTarget().removeIncomingEdge(edge);
			if(edge.getTarget().getIncomingEdges().isEmpty()){
				// remove this node
				this.removeNode(edge.getTarget());
			}
		}
		tree.removeEdge(edge);
	}
	
	private void removeNode(Node node){
		for(Edge e: node.getIncomingEdges()){
			removeEdge(e, node);
		}
		if(node instanceof Block){
			for(Edge e: ((Block)node).getOutgoingEdges()){
				removeEdge(e, node);
			}
		}
		// also remove the root if this node is the root
		if(tree.getRoot().equals(node)){
			tree.setRoot(null);
		}
		tree.removeNode(node);
	}


}
