package org.processmining.processtree.editor.graphpanel;

import java.awt.Dimension;
import java.awt.event.MouseListener;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.UUID;

import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JScrollPane;

import org.jgraph.JGraph;
import org.processmining.processtree.ProcessTree;
import org.processmining.processtree.editor.editorpanel.EditorPanel;
import org.processmining.processtree.editor.interaction.GraphMouseListener;
import org.processmining.processtree.visualization.tree.TreeLayoutBuilder;

public class GraphPanel {
	ProcessTree tree;
	JGraph graph;
	JFrame frame;
	Dimension dimGraphPanel;
	TreeLayoutBuilder builder;
	JScrollPane scrollPane;
	List<MouseListener> mouseListeners;
	EditorPanel editorPanel;
	Set<UUID> foldedNodes;
	
	public GraphPanel(ProcessTree tree, Dimension dimGraphPanel, Set<UUID> foldedNodes){
		this.tree = tree;
		this.mouseListeners = new ArrayList<>();
		this.dimGraphPanel = dimGraphPanel;
		this.frame = new JFrame();
		this.foldedNodes = foldedNodes;
		redraw();
	}
	
	public JComponent getComponent(){
		return (JComponent) frame.getContentPane();
	}
	
	public JGraph getGraph(){
		return graph;
	}
	
	public void addMouseListener(MouseListener mouseListener){
		this.mouseListeners.add(mouseListener);
		graph.addMouseListener(mouseListener);
	}
	
	public void setEditorPanel(EditorPanel editorPanel){
		this.editorPanel = editorPanel;
	}
		
	public void redraw(){
		if(editorPanel != null){
			editorPanel.freezePanel();
		}
		int verticalValue = 0;
		int horizontalValue = 0;
		if(scrollPane != null){
			frame.getContentPane().remove(scrollPane);
			verticalValue = this.scrollPane.getVerticalScrollBar().getValue();
			horizontalValue = this.scrollPane.getHorizontalScrollBar().getValue();
		}
		this.builder = new TreeLayoutBuilder(tree, null, null, foldedNodes);
		this.graph = builder.getJGraph();
		this.graph.setEditable(false);
		this.graph.setMoveable(false);
		for(MouseListener mouseListen: mouseListeners){
			graph.addMouseListener(mouseListen);
		}
		
		for(MouseListener mouseListen: mouseListeners){
			if(mouseListen instanceof GraphMouseListener){
				((GraphMouseListener)mouseListen).updateGraph(graph);
			}
		}
		this.scrollPane = new JScrollPane(graph);
		//this.scrollPane.setPreferredSize(dimGraphPanel);
		scrollPane.getVerticalScrollBar().setValue(verticalValue);
		scrollPane.getVerticalScrollBar().setValue(verticalValue);
		scrollPane.getHorizontalScrollBar().setValue(horizontalValue);
		scrollPane.getHorizontalScrollBar().setValue(horizontalValue);
		frame.getContentPane().add(scrollPane);
		//frame.setPreferredSize(dimGraphPanel);
		//frame.getContentPane().setPreferredSize(dimGraphPanel);
		frame.getContentPane().validate();
		if(editorPanel != null){
			editorPanel.defrostPanel();
		}
		// now fire a selection on the mouselistener
		for(MouseListener mouseListen: mouseListeners){
			if(mouseListen instanceof GraphMouseListener){
				((GraphMouseListener)mouseListen).mousePressed(null);
			}
		}
	}
}
