package org.processmining.processtree.editor.propertypanel;

import java.awt.Dimension;
import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

import javax.swing.JComponent;
import javax.swing.JTabbedPane;

import org.processmining.plugins.properties.processmodel.visualisation.VisualiseProperties;
import org.processmining.processtree.Edge;
import org.processmining.processtree.Node;
import org.processmining.processtree.ProcessTree;
import org.processmining.processtree.ProcessTreeElement;
import org.processmining.processtree.Task.Manual;
import org.processmining.processtree.editor.graphpanel.GraphPanel;
import org.processmining.processtree.visualization.data.DataVisualiser;
import org.processmining.processtree.visualization.originator.OriginatorVisualiser;


public class PropertyPanel {
	ProcessTree tree;
	JTabbedPane tabPane;
	Dimension dimPropertyPanel;
	UUID lastSelectedID;
	GraphPanel graphPanel;
		
	public PropertyPanel(ProcessTree tree, Dimension dimPropertyPanel){
		this.tree = tree;
		this.tabPane = new JTabbedPane();
		this.dimPropertyPanel = dimPropertyPanel;
		this.lastSelectedID = null;
		//panel.setPreferredSize(dimPropertyPanel);
	}
	
	public void setGraphPanel(GraphPanel graphPanel){
		this.graphPanel = graphPanel;
	}
	
	public JComponent getComponent(){
		return tabPane;
	}
	
	public void mousePressed(UUID id){
		if(id != null){
			String lastName = "";
			if(id.equals(lastSelectedID)){
				// we have it again keep the ordering in the tab
				if(tabPane.getSelectedComponent() != null){
					lastName = tabPane.getSelectedComponent().getName();
				}
			}
			lastSelectedID = id;
			while(tabPane.getTabCount() > 0){
				tabPane.removeTabAt(0);
			}
			// now create an overview of the different things
			// first get the node
			ProcessTreeElement elem = tree.getElement(id);
			// now make the different properties visible
			// variables
			List<JComponent> components = new ArrayList<JComponent>();
			try {
				components = VisualiseProperties.visualiseProperties(elem, true);
			} catch (Exception ex) {
				ex.printStackTrace();
			}
			if(id != null && id.equals(tree.getID())){
				// also show the data perspective
				DataVisualiser dataVis = new DataVisualiser(tree, true);
				try {
					components.addAll(dataVis.dataVisualiser());
				} catch (Exception ex) {
					ex.printStackTrace();
				}
				OriginatorVisualiser orgVis = new OriginatorVisualiser(tree, true);
				try {
					components.addAll(orgVis.originatorVisualiser());
				} catch (Exception ex) {
					ex.printStackTrace();
				}
			}
			if(elem instanceof Edge){
				DataVisualiser dataVis = new DataVisualiser(tree, null, (Edge) elem, true, graphPanel);
				try {
					components.addAll(dataVis.dataVisualiser());
				} catch (InstantiationException | IllegalAccessException e) {
					e.printStackTrace();
				}
			}
			if(elem instanceof Node){
				DataVisualiser dataVis = new DataVisualiser(tree, (Node)elem, null, true, graphPanel);
				try {
					components.addAll(dataVis.dataVisualiser());
				} catch (InstantiationException | IllegalAccessException e) {
					e.printStackTrace();
				}
				
				if(elem instanceof Manual){
					OriginatorVisualiser orgVis = new OriginatorVisualiser(tree, (Manual)elem, true);
					try {
						components.addAll(orgVis.originatorVisualiser());
					} catch (InstantiationException | IllegalAccessException e) {
						e.printStackTrace();
					}
				}
			}
			JComponent focusComponent = null;
			for(JComponent comp: components){
				tabPane.addTab(comp.getName(), comp);
				if(comp.getName().equals(lastName)){
					focusComponent = comp;
				}
			}
			if(focusComponent != null){
				tabPane.setSelectedComponent(focusComponent);
			}
			tabPane.validate();
		}
	}
	
}
