package org.processmining.processtree.impl;

import java.util.Collection;
import java.util.HashSet;
import java.util.Set;
import java.util.UUID;

import org.processmining.processtree.Expression;
import org.processmining.processtree.Variable;

public class ExpressionImpl extends ProcessTreeElementImpl implements Expression {

	protected final String expression;
	protected final Set<Variable> variables;

	public boolean equals(Object o){
		if(super.equals(o)){
			return true;
		}
		else if(o instanceof Expression){
			return this.getExpression().equalsIgnoreCase(((Expression)o).getExpression()) && !EdgeImpl.NOEXPRESSION.equals(o);
		}
		return false;
	}
	
	public ExpressionImpl(String expression, Variable... variables) {
		this(UUID.randomUUID(), expression, variables);
	}
	
	public ExpressionImpl(UUID id, String expression, Variable... variables) {
		this(id, id.toString(), expression, variables);
	}
	
	public ExpressionImpl(String name, String expression, Variable...variables){
		this(UUID.randomUUID(), name, expression, variables);
	}
	
	public ExpressionImpl(UUID id, String name, String expression, Variable... variables){
		super(id, name);
		this.expression = expression;
		this.variables = new HashSet<Variable>();
		for (Variable var : variables) {
			this.variables.add(var);
		}
	}
	
	public ExpressionImpl(Expression e){
		super(e);
		this.expression = e.getExpression();
		this.variables = new HashSet<Variable>();
	}

	@Override
	public String getExpression() {
		return expression;
	}

	@Override
	public Collection<Variable> getVariables() {
		return variables;
	}

	public boolean addVariable(Variable var) {
		return variables.add(var);
	}

	public boolean removeVariable(Variable var) {
		return variables.remove(var);
	}

	public String toString() {
		return "[" + expression + "]";
	}
}
