package org.processmining.processtree.ptml;

import java.util.Map;
import java.util.UUID;

import org.processmining.processtree.Edge;
import org.processmining.processtree.Expression;
import org.processmining.processtree.ProcessTree;
import org.xmlpull.v1.XmlPullParser;

public class PtmlIsExpression extends PtmlEdge {
	public final static String TAG = "usesExpression";
	boolean removable = false;
		
	public PtmlIsExpression(UUID id) {
		super(TAG, id);
		removable = false;
	}
	
	protected void importAttributes(XmlPullParser xpp, Ptml ptml) {
		super.importAttributes(xpp, ptml);
		importRemovable(xpp, ptml);
	}

	private void importRemovable(XmlPullParser xpp, Ptml ptml) {
		String value = xpp.getAttributeValue(null, "removable");
		if (value != null) {
			//expression = new String(value);
			removable = Boolean.parseBoolean(value);
		}
		ptml.log(value != null ? value : "<no value>");
	}
	
	protected String exportAttributes(Ptml ptml) {
		return super.exportAttributes(ptml) + exportRemovable(ptml);
	}

	private String exportRemovable(Ptml ptml) {
		if (removable) {
			return exportAttribute("removable", "true", ptml);
		}
		return "";
	}

	public void unmarshall(ProcessTree tree, Map<UUID, Edge> edgeMap, Map<UUID, Expression> expressionMap) {
		Edge edge = edgeMap.get(sourceId);
		Expression expression = expressionMap.get(targetId);
		if (edge != null && expression != null) {
			edge.getExpressions().add(expression);
			if(removable){
				edge.getRemovableExpressions().add(expression);
			}
		}
	}
	
	public void marshall(ProcessTree tree, Edge edge, Expression expression, Map<Expression, UUID> expressionMap) {
		sourceId = edge.getID();
		targetId = expressionMap.get(expression);
		if(edge.getRemovableExpressions() != null){
			removable = edge.getRemovableExpressions().contains(expression);
		}
	}

}
