package org.processmining.processtree.ptml;

import java.util.Map;
import java.util.UUID;

import org.processmining.processtree.Node;
import org.processmining.processtree.ProcessTree;
import org.processmining.processtree.Variable;
import org.xmlpull.v1.XmlPullParser;

public class PtmlIsWritten extends PtmlEdge {

	public final static String TAG = "writesVariable";
	boolean removable;
	
	public PtmlIsWritten(UUID id) {
		super(TAG, id);
		removable = false;
	}
	
	protected void importAttributes(XmlPullParser xpp, Ptml ptml) {
		super.importAttributes(xpp, ptml);
		importRemovable(xpp, ptml);
	}

	private void importRemovable(XmlPullParser xpp, Ptml ptml) {
		String value = xpp.getAttributeValue(null, "removable");
		if (value != null) {
			//expression = new String(value);
			removable = Boolean.parseBoolean(value);
		}
		ptml.log(value != null ? value : "<no value>");
	}
	
	protected String exportAttributes(Ptml ptml) {
		return super.exportAttributes(ptml) + exportRemovable(ptml);
	}

	private String exportRemovable(Ptml ptml) {
		if (removable) {
			return exportAttribute("removable", "true", ptml);
		}
		return "";
	}

	public void unmarshall(ProcessTree tree, Map<UUID, Node> nodeMap, Map<UUID, Variable> variableMap) {
		Node node = nodeMap.get(sourceId);
		Variable variable = variableMap.get(targetId);
		if (node != null && variable != null) {
			node.getWrittenVariables().add(variable);
			if(removable){
				node.getRemovableWrittenVariables().add(variable);
			}
		}
	}

	public void marshall(ProcessTree tree, Node node, Variable variable,
			Map<Variable, UUID> variableMap) {
		sourceId = node.getID();
		targetId = variableMap.get(variable);
		if(node.getRemovableWrittenVariables() != null){
			removable = node.getRemovableWrittenVariables().contains(variable);
		}
	}
}