package org.processmining.processtree.test;

import org.processmining.processtree.Block;
import org.processmining.processtree.Edge;
import org.processmining.processtree.Event;
import org.processmining.processtree.Expression;
import org.processmining.processtree.ProcessTree;
import org.processmining.processtree.Task;
import org.processmining.processtree.Variable;
import org.processmining.processtree.impl.AbstractBlock;
import org.processmining.processtree.impl.AbstractEvent;
import org.processmining.processtree.impl.AbstractOriginator.Group;
import org.processmining.processtree.impl.AbstractOriginator.Role;
import org.processmining.processtree.impl.AbstractTask;
import org.processmining.processtree.impl.ExpressionImpl;
import org.processmining.processtree.impl.ProcessTreeImpl;
import org.processmining.processtree.impl.VariableImpl;

public class TestProcessTree {

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		ProcessTree tree = getTestVisTree();

		System.out.println(tree);
		System.out.println("is Tree: " + tree.isTree());
		System.out.println("is DAG:  " + tree.isDag());

	}

	public static ProcessTree getTree() {
		//return getSimpleTreeWithSkips();
		return getTestVisTree();
	}

	public static ProcessTree getTestVisTree() {
		ProcessTree tree = new ProcessTreeImpl();
		Edge edge;

		// And( Xor( F+complete ,  ) , B+complete , A+complete )

		Block and = new AbstractBlock.And("And");
		and.setProcessTree(tree);
		tree.addNode(and);
		tree.setRoot(and);

		Block xor = new AbstractBlock.Xor("xor");
		xor.setProcessTree(tree);
		tree.addNode(xor);

		edge = and.addChild(xor);
		xor.addIncomingEdge(edge);
		tree.addEdge(edge);

		Task f = new AbstractTask.Manual("F");
		f.setProcessTree(tree);
		tree.addNode(f);

		edge = xor.addChild(f);
		f.addIncomingEdge(edge);
		tree.addEdge(edge);

		Task tau = new AbstractTask.Automatic("");
		tau.setProcessTree(tree);
		tree.addNode(tau);

		edge = xor.addChild(tau);
		tau.addIncomingEdge(edge);
		tree.addEdge(edge);

		Task a = new AbstractTask.Manual("B");
		a.setProcessTree(tree);
		tree.addNode(a);

		edge = and.addChild(a);
		a.addIncomingEdge(edge);
		tree.addEdge(edge);

		Task b = new AbstractTask.Manual("A");
		b.setProcessTree(tree);
		tree.addNode(b);

		edge = and.addChild(b);
		b.addIncomingEdge(edge);
		tree.addEdge(edge);

		return tree;
	}

	public static ProcessTree getSimpleTreeWithSkips() {
		ProcessTree tree = new ProcessTreeImpl();
		Edge edge;

		Block and = new AbstractBlock.And("N1");
		and.setProcessTree(tree);
		tree.addNode(and);
		tree.setRoot(and);

		for (int i = 0; i < 10; i++) {
			Task a;
			if (i % 3 == 0) {
				a = new AbstractTask.Automatic("task " + i);
			} else {
				a = new AbstractTask.Manual("task " + i);
			}
			a.setProcessTree(tree);
			tree.addNode(a);

			edge = and.addChild(a);
			a.addIncomingEdge(edge);
			tree.addEdge(edge);

		}
		return tree;

	}

	public static ProcessTree getSimpleTree() {
		ProcessTree tree = new ProcessTreeImpl();
		Edge edge;

		// xor loop
		Block loopXor = new AbstractBlock.XorLoop("N1");
		loopXor.setProcessTree(tree);
		tree.addNode(loopXor);
		tree.setRoot(loopXor);

		// Leaf A
		Task a = new AbstractTask.Manual("A");
		a.setProcessTree(tree);
		tree.addNode(a);

		edge = loopXor.addChild(a);
		a.addIncomingEdge(edge);
		tree.addEdge(edge);

		// Leaf B
		Task b = new AbstractTask.Manual("B");
		b.setProcessTree(tree);
		tree.addNode(b);

		edge = b.addParent(loopXor);
		loopXor.addOutgoingEdge(edge);
		tree.addEdge(edge);

		// Leaf A again
		edge = loopXor.addChild(a);
		a.addIncomingEdge(edge);
		tree.addEdge(edge);

		return tree;
	}

	public static ProcessTree getExampleTree() {
		ProcessTree tree = new ProcessTreeImpl();

		// Roles
		Role receptionist = new Role("receptionise");
		tree.addOriginator(receptionist);

		Role advisor = new Role("advisor");
		tree.addOriginator(advisor);

		Role clerc = new Role("clerc");
		tree.addOriginator(clerc);

		Role scientist = new Role("scientist");
		tree.addOriginator(scientist);

		// Groups
		Group frontOffice = new Group("front office");
		tree.addOriginator(frontOffice);

		Group backOffice = new Group("back office");
		tree.addOriginator(backOffice);

		Group finance = new Group("finance");
		tree.addOriginator(finance);

		Group rAndD = new Group("R&D");
		tree.addOriginator(rAndD);

		// variables
		Variable d1 = new VariableImpl("d1");
		tree.addVariable(d1);

		Variable d2 = new VariableImpl("d2");
		tree.addVariable(d2);

		Variable d3 = new VariableImpl("d3");
		tree.addVariable(d3);

		Variable d4 = new VariableImpl("d4");
		tree.addVariable(d4);

		//Expressions
		Expression d1GTd2 = new ExpressionImpl("d1 > d2", d1, d2);

		//Nodes

		//root
		Block root = new AbstractBlock.Seq("N0");
		root.getReadVariables().add(d1);
		root.getWrittenVariables().add(d4);
		root.setProcessTree(tree);
		tree.addNode(root);
		tree.setRoot(root);

		// xor loop
		Block loopXor = new AbstractBlock.XorLoop("N1");
		loopXor.setProcessTree(tree);
		loopXor.getReadVariables().add(d1);
		loopXor.getWrittenVariables().add(d1);
		loopXor.getWrittenVariables().add(d2);
		tree.addNode(loopXor);

		Edge edge = root.addChild(loopXor, d1GTd2);
		loopXor.addIncomingEdge(edge);
		tree.addEdge(edge);

		// Leaf A
		Task a = new AbstractTask.Manual("A", receptionist, frontOffice);
		a.setProcessTree(tree);
		a.getReadVariables().add(d1);
		a.getWrittenVariables().add(d1);
		a.getWrittenVariables().add(d2);
		tree.addNode(a);

		edge = loopXor.addChild(a);
		a.addIncomingEdge(edge);
		tree.addEdge(edge);

		// Leaf B
		Task b = new AbstractTask.Manual("B", receptionist, backOffice);
		b.setProcessTree(tree);
		b.getReadVariables().add(d2);
		b.getWrittenVariables().add(d2);
		tree.addNode(b);

		edge = b.addParent(loopXor);
		loopXor.addOutgoingEdge(edge);
		tree.addEdge(edge);

		// Leaf A again
		edge = loopXor.addChild(a);
		a.addIncomingEdge(edge);
		tree.addEdge(edge);

		// Deferred choice in sequence
		Block def = new AbstractBlock.Def("N2");
		def.setProcessTree(tree);
		def.getWrittenVariables().add(d3);
		def.getWrittenVariables().add(d4);
		tree.addNode(def);

		edge = root.addChild(def);
		edge.setBlockable(true);
		edge.setHideable(true);
		def.addIncomingEdge(edge);
		tree.addEdge(edge);

		// Event timeout
		Event timeout = new AbstractEvent.TimeOut("timeout", "Timeout occurred");
		timeout.setProcessTree(tree);
		timeout.getWrittenVariables().add(d3);
		timeout.getWrittenVariables().add(d4);
		tree.addNode(timeout);

		edge = def.addChild(timeout);
		timeout.addIncomingEdge(edge);
		tree.addEdge(edge);

		// Event message
		Event message = new AbstractEvent.Message("message", "Message received");
		message.setProcessTree(tree);
		message.getWrittenVariables().add(d3);
		message.getWrittenVariables().add(d4);
		tree.addNode(message);

		edge = def.addChild(message);
		message.addIncomingEdge(edge);
		tree.addEdge(edge);

		// Leaf skip
		Task skip = new AbstractTask.Automatic("skip");
		skip.setProcessTree(tree);
		//skip.getWrittenVariables().add(d3);
		//skip.getWrittenVariables().add(d4);
		tree.addNode(skip);

		edge = timeout.addChild(skip);
		skip.addIncomingEdge(edge);
		tree.addEdge(edge);

		// Leaf C
		Task c = new AbstractTask.Manual("C", advisor, finance);
		c.setProcessTree(tree);
		c.getReadVariables().add(d3);
		c.getWrittenVariables().add(d3);
		c.getWrittenVariables().add(d4);
		tree.addNode(c);

		edge = message.addChild(c);
		c.addIncomingEdge(edge);
		tree.addEdge(edge);

		// Leaf A again, but noe before the def
		edge = root.addChildAt(a, 1);
		a.addIncomingEdge(edge);
		a.addParent(root);
		tree.addEdge(edge);

		// Placeholder
		Block placeHolder = new AbstractBlock.PlaceHolder("placeholder");
		placeHolder.setProcessTree(tree);
		placeHolder.getReadVariables().add(d1);
		placeHolder.getReadVariables().add(d3);
		placeHolder.getWrittenVariables().add(d4);
		tree.addNode(placeHolder);

		edge = root.addChild(placeHolder);
		edge.setHideable(true);
		edge.setBlockable(true);
		placeHolder.addIncomingEdge(edge);
		tree.addEdge(edge);

		// First child of placeholder
		edge = placeHolder.addChild(def);
		def.addIncomingEdge(edge);
		tree.addEdge(edge);

		// Leaf D;
		Task d = new AbstractTask.Manual("D", scientist, rAndD);
		d.setProcessTree(tree);
		d.getReadVariables().add(d1);
		d.getWrittenVariables().add(d4);
		tree.addNode(d);

		edge = placeHolder.addChild(d);
		d.addIncomingEdge(edge);
		tree.addEdge(edge);

		// Leaf E;
		Task e = new AbstractTask.Manual("E", clerc, finance);
		e.setProcessTree(tree);
		e.getReadVariables().add(d3);
		e.getWrittenVariables().add(d4);
		tree.addNode(e);

		edge = placeHolder.addChild(e);
		e.addIncomingEdge(edge);
		tree.addEdge(edge);

		return tree;
	}
}
