package org.processmining.processtree.visualization.interaction;

import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.UUID;

import javax.swing.JComponent;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;

import org.jgraph.JGraph;
import org.jgraph.graph.DefaultGraphCell;
import org.processmining.plugins.properties.processmodel.visualisation.VisualiseProperties;
import org.processmining.processtree.ProcessTree;
import org.processmining.processtree.ProcessTreeElement;
import org.processmining.processtree.visualization.data.DataVisualiser;
import org.processmining.processtree.visualization.originator.OriginatorVisualiser;
import org.processmining.processtree.visualization.tree.TreeLayoutBuilder;

public class MouseInteraction extends MouseAdapter{
	ProcessTree tree;
	JGraph graph;
	TreeLayoutBuilder builder;
	JSplitPane splitPane;
	JScrollPane scrollPane;
	JTabbedPane tabPane;
	Set<UUID> foldedNodes;
	UUID lastElement;
	Long lastClick;
	Long clickInterval;
	
	public MouseInteraction(ProcessTree tree, JGraph graph, JTabbedPane tabPane){
		this(tree, graph, null, null, null, tabPane, new HashSet<UUID>());
	}
	
	public MouseInteraction(ProcessTree tree, JGraph graph, TreeLayoutBuilder builder, JSplitPane splitPane, JScrollPane scrollPane, JTabbedPane tabPane, Set<UUID> foldedNodes){
		this.tree = tree;
		this.graph = graph;
		this.builder = builder;
		this.tabPane = tabPane;
		this.splitPane = splitPane;
		this.scrollPane = scrollPane;
		this.foldedNodes = foldedNodes;
		this.lastElement = null;
		this.lastClick = null;
		this.clickInterval = 2l * 1000l * 1000l * 1000l;
	}
	
	public void mousePressed(MouseEvent e){
		Long systemTime = System.nanoTime();
		boolean doubleClick = false;
		// get the cell
		int x = e.getX();
		int y = e.getY();
		DefaultGraphCell cell = (DefaultGraphCell) graph.getFirstCellForLocation(x, y);
		UUID id;
		if(cell != null){
			id = (UUID) cell.getAttributes().get(TreeLayoutBuilder.ID);
		}
		else{
			id = tree.getID();
		}
		if(e.getButton() != MouseEvent.BUTTON1 && lastClick != null && lastClick - systemTime < clickInterval && id != null && lastElement != null && id.equals(lastElement)){
			// we have a double click
			if(foldedNodes.contains(id)){
				foldedNodes.remove(id);
			}
			else{
				foldedNodes.add(id);
			}
			doubleClick = true;
			if(builder != null && splitPane != null){
				splitPane.removeAll();
				builder.reset();
				graph = builder.getJGraph();
				graph.addMouseListener(this);
				scrollPane = new JScrollPane(graph);
				splitPane.add(scrollPane);
				splitPane.add(tabPane);
				splitPane.validate();
			}
		}
			
		while(tabPane.getTabCount() > 0){
			tabPane.removeTabAt(0);
		}
		// now create an overview of the different things
		// first get the node
		ProcessTreeElement elem = tree.getElement(id);
		// now make the different properties visible
		// variables
		List<JComponent> components = new ArrayList<JComponent>();
		try {
			components = VisualiseProperties.visualiseProperties(elem);
		} catch (Exception ex) {
			// TODO Auto-generated catch block
			ex.printStackTrace();
		}
		if(id != null && id.equals(tree.getID())){
			// also show the data perspective
			DataVisualiser dataVis = new DataVisualiser(tree);
			try {
				components.addAll(dataVis.dataVisualiser());
			} catch (Exception ex) {
				ex.printStackTrace();
			}
			OriginatorVisualiser orgVis = new OriginatorVisualiser(tree);
			try {
				components.addAll(orgVis.originatorVisualiser());
			} catch (Exception ex) {
				// TODO Auto-generated catch block
				ex.printStackTrace();
			}
		}
		for(JComponent comp: components){
			tabPane.addTab(comp.getName(), comp);
		}
		if(!doubleClick){
			lastClick = System.nanoTime();
			lastElement = id;
		}
		else{
			lastClick = null;
			lastElement = null;
		}
	}
}
