package org.processmining.processtree.visualization.jgraph;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.geom.AffineTransform;
import java.awt.geom.Ellipse2D;
import java.awt.geom.GeneralPath;

import org.jgraph.graph.CellView;
import org.jgraph.graph.DefaultGraphCell;

import com.jgraph.components.labels.MultiLineVertexRenderer;

public class EventRenderer extends MultiLineVertexRenderer {

	public static enum EventType {
		TIMEOUT("TO"), MESSAGE("Mes");
		private final String s;

		EventType(String s) {
			this.s = s;
		}

		public String toString() {
			return s;
		}
	};

	/**
	 * 
	 */
	private static final long serialVersionUID = 4910262989128580150L;
	protected EventType type;

	@Override
	protected void paintComponent(Graphics g) {

		Graphics2D g2d = (Graphics2D) g;

		GeneralPath gatewayDecorator = new GeneralPath();
		double s = (getWidth() - 2.0 * borderWidth) / 33.;
		switch (type) {
			case MESSAGE :
				drawMessage(g2d, gatewayDecorator, s);
				break;
			case TIMEOUT :
				drawTimeout(g2d, gatewayDecorator, s);
				break;
		}
		//super.paintComponent(g);

		AffineTransform at = new AffineTransform();
		//at.scale(s, s);
		//gatewayDecorator.transform(at);
		//at = new AffineTransform();
		at.translate(borderWidth, borderWidth);
		gatewayDecorator.transform(at);

		g.setColor(Color.BLACK);
		g2d.draw(gatewayDecorator);
	}

	protected void drawTimeout(Graphics2D g2d, GeneralPath gatewayDecorator, double s) {
		BPMNOperatorRenderer.parseSVG(g2d, gatewayDecorator, "M 17 8 L 16 16 L 24 16", s);
		gatewayDecorator.append(new Ellipse2D.Double(s * 5, s * 5, s * 22, s * 22), false);

		g2d.setStroke(new BasicStroke(1F));

	}

	protected void drawMessage(Graphics2D g2d, GeneralPath gatewayDecorator, double s) {
		BPMNOperatorRenderer.parseSVG(g2d, gatewayDecorator,
				"M 5 8.5 L 27 8.5 L 27 23.5 L 5 23.3 L 5 8.5 L 16 16 L 27 8.5 ", s);
		g2d.setStroke(new BasicStroke(1F));

	}

	public void installAttributes(CellView view) {
		super.installAttributes(view);
		type = (EventType) ((DefaultGraphCell) view.getCell()).getUserObject();
	}

}
