package org.processmining.processtree.visualization.jgraph;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Stroke;
import java.awt.geom.AffineTransform;
import java.awt.geom.Ellipse2D;
import java.awt.geom.GeneralPath;

import org.jgraph.graph.CellView;
import org.jgraph.graph.DefaultGraphCell;
import org.processmining.processtree.ProcessTree;
import org.processmining.processtree.ProcessTree.Type;

import com.jgraph.components.labels.MultiLineVertexRenderer;

public class TreeOperatorRenderer extends MultiLineVertexRenderer {

	private static final long serialVersionUID = 4910262989128580150L;
	protected ProcessTree.Type type;

	private float[] pattern = new float[] { 5, 5 };

	@Override
	protected void paintShapeBorder(Graphics g) {
		Graphics2D g2d = (Graphics2D) g;
		if (type == Type.PLACEHOLDER) {
			Stroke s = g2d.getStroke();
			g2d.setStroke(new BasicStroke(borderWidth, BasicStroke.CAP_SQUARE, BasicStroke.JOIN_MITER, 10.0F, pattern,
					0F));
			super.paintShapeBorder(g);
			g2d.setStroke(s);
		} else {
			super.paintShapeBorder(g);
		}
	}

	@Override
	protected void paintComponent(Graphics g) {
		Graphics2D g2d = (Graphics2D) g;

		GeneralPath gatewayDecorator = new GeneralPath();
		String path = "";
		double s = (getWidth() - 2.0 * borderWidth) / 33.;
		switch (type) {
			case AND :
				path = "M 8.5 15 L 8.5 17 L 15 17 L 15 23.5 L 17 23.5 L 17 17 L 23.5 17 L 23.5 15 L 17 15 L 17 8.5 L 15 8.5 L 15 15 L 8.5 15";
				break;
			case OR :
				path = "M 8.5,16 L 23.5,16 M 16,8.5 L 16,23.5 M 10.5,10.5 L 21.5,21.5 M 10.5,21.5 L 21.5,10.5";
				break;
			case LOOPDEF :
				g2d.drawArc((int) (5 * s), (int) (5 * s), (int) (25 * s), (int) (25 * s), -30, 240);
				path += "M 26 24 L 31 19 L 25 17.5 L 26 24 ";
				//$FALL-THROUGH$
			case DEF :
				gatewayDecorator.append(new Ellipse2D.Double(s * 5.5, s * 5.5, s * 21, s * 21), false);
				gatewayDecorator.append(new Ellipse2D.Double(s * 7.5, s * 7.5, s * 17, s * 17), false);
				path += "M 16 10 L 21.7 14.15 L 19.5 20.85 L 12.5 20.85 L 10.3 14.15 L 16 10";
				break;
			case LOOPXOR :
				g2d.drawArc((int) (5 * s), (int) (5 * s), (int) (25 * s), (int) (25 * s), -30, 240);
				path += "M 26 24 L 31 19 L 25 17.5 L 26 24 ";
				//$FALL-THROUGH$
			case XOR :
				path += "M 11.5 9.5 L 13.5 9.5 L 20.5 22.5 L 18.5 22.5 L 11.5 9.5 M 11.5 22.5 L 18.5 9.5 L 20.5 9.5 L 13.5 22.5 L 11.5 22.5";
				break;
			case MESSAGE :
				path = "M 5 8.5 L 27 8.5 L 27 23.5 L 5 23.3 L 5 8.5 L 16 16 L 27 8.5 ";
				break;
			case SEQ :
				path = "M 4 14 L 22 14 L 22 11 L 30 16.5 L 22 22 L 22 19 L 4 19 L 4 14 ";
				break;
			case TIMEOUT :
				path = "M 17 8 L 16 16 L 24 16";
				break;
			case PLACEHOLDER :
			case AUTOTASK :
			case MANTASK :
				break;
		}
		BPMNOperatorRenderer.parseSVG(g2d, gatewayDecorator, path, s);
		//super.paintComponent(g);

		AffineTransform at = new AffineTransform();
		//at.scale(s, s);
		//gatewayDecorator.transform(at);
		//at = new AffineTransform();
		at.translate(borderWidth, borderWidth);
		gatewayDecorator.transform(at);

		g.setColor(Color.BLACK);

		if (type == Type.XOR || type == Type.LOOPXOR || type == Type.AND || type == Type.SEQ) {
			g2d.fill(gatewayDecorator);
		} else {
			g2d.draw(gatewayDecorator);
		}

	}

	public void installAttributes(CellView view) {
		super.installAttributes(view);
		type = (Type) ((DefaultGraphCell) view.getCell()).getUserObject();
	}

}
