package org.processmining.processtree.visualization.originator;

import info.clearthought.layout.TableLayout;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;

import org.processmining.framework.util.Pair;
import org.processmining.plugins.properties.processmodel.visualisation.VisualiseProperties;
import org.processmining.processtree.Node;
import org.processmining.processtree.Originator;
import org.processmining.processtree.ProcessTree;
import org.processmining.processtree.Task.Manual;

public class OriginatorVisualiser {
	ProcessTree tree;
	Manual manual;
	Boolean editable;
	Map<Pair<Manual, Originator>, OriginatorCell> originatorCells;
	
	public OriginatorVisualiser(ProcessTree tree){
		this(tree, false);
	}
	
	public OriginatorVisualiser(ProcessTree tree, Boolean editable){
		this(tree, null, editable);
	}
	
	public OriginatorVisualiser(ProcessTree tree, Manual manual, Boolean editable){
		this.tree = tree;
		this.manual = manual;
		this.editable = editable;
		this.originatorCells = new HashMap<>();
	}
	
	public List<JComponent> originatorVisualiser() throws InstantiationException, IllegalAccessException{
		List<JComponent> comps = new ArrayList<JComponent>();
		if(manual == null){
			comps.add(originators());
			comps.add(originatorProperties());
		}
		else{
			comps.add(originators(manual));
			comps.add(originatorProperties(manual));
		}
		return comps;
	}
		
	public JComponent originators(){
		return originators(null);
	}
	
	public JComponent originators(Manual manual){
		// now let's do it properly
		JPanel panel = new JPanel();
		int width = tree.getOriginators().size() + 1;
		int height = 0;
		for(Node n: tree.getNodes()){
			if(n instanceof Manual){
				height++;
			}
		}
		height = 2 * height + 1;
		double[][] layout = new double[2][Math.max(width, height)];
		for(int i = 0; i < width; i++){
			layout[0][i] = 100;
		}
		for(int i = 0; i < height; i++){
			layout[1][i] = 30;
		}
		panel.setLayout(new TableLayout(layout));
		// collect the elements as the ordering is not given
		List<Manual> manuals = new ArrayList<>();
		for(Node n: tree.getNodes()){
			if(n instanceof Manual){
				manuals.add((Manual) n);
			}
		}
		if(manual != null){
			manuals.clear();
			manuals.add(manual);
		}
		List<Originator> orgs = new ArrayList<>();
		for(Originator org: tree.getOriginators()){
			orgs.add(org);
		}
		
		// generate the overview
		int row = 0;
		int offset = 1;
		for(Manual m: manuals){
			// put this node on the panel
			JLabel label = new JLabel(m.getName());
			panel.add(label, "0, " + (row + offset) + ", 0, " + (row + 1 + offset) + ", c, c");
			row += 2;
		}
		int column = 0;
		offset = 1;
		for(Originator org: orgs){
			JLabel label = new JLabel(org.getName());
			panel.add(label, "" + (column + offset) + ", 0, " + (column + offset) + " 0 , l, c");
			column += 1;
		}
		// now render the combinations
		int offsetRow = 1;
		int offsetColumn = 1;
		row = 0;
		column = 0;
		for(Manual man: manuals){
			for(Originator org: orgs){
				renderCellVariable(panel, man, org, offsetRow + row, offsetColumn + column);
				column += 1;
			}
			row += 2;
			column = 0;
		}
		JScrollPane scroll = new JScrollPane(panel);
		scroll.setName("Originators");
		return scroll;
	}
	
	private void renderCellVariable(JComponent component, Manual man, Originator org, int lowerRow, int lowerColumn){
		JCheckBox executes = new JCheckBox("o", man.getOriginators().contains(org));
		executes.setToolTipText("Originator");
		JCheckBox remExecutes = new JCheckBox("ro", man.getRemovableOriginators().contains(org));
		remExecutes.setToolTipText("Removable Originator");
		executes.setEnabled(editable);
		executes.addActionListener(new OriginatorApplyActionListener(originatorCells));
		remExecutes.setEnabled(executes.isSelected() && editable);
		remExecutes.addActionListener(new OriginatorApplyActionListener(originatorCells));
		OriginatorCell orgCell = new OriginatorCell(executes, remExecutes);
		originatorCells.put(new Pair<Manual, Originator>(man, org), orgCell);
		// now add the nodes to the component
		component.add(executes, "" + lowerColumn + ", " + lowerRow + ", " + lowerColumn + ", " + lowerRow + ", l, c");
		component.add(remExecutes, "" + lowerColumn + ", " + (lowerRow + 1) + ", " + lowerColumn + ", " + (lowerRow + 1) + ", l, c");
	}
	
	private JComponent originatorProperties() throws InstantiationException, IllegalAccessException{
		return originatorProperties(null);
	}
	
	private JComponent originatorProperties(Manual manual) throws InstantiationException, IllegalAccessException{
		// set the variable properties
		JTabbedPane tabPaneSuper = new JTabbedPane();
		tabPaneSuper.setName("OriginatorProperties");
		for(Originator org: tree.getOriginators()){
			if(manual != null && !manual.getOriginators().contains(org) && !manual.getRemovableOriginators().contains(org)){
				// never mind
			}
			else{
				JTabbedPane tabpane = new JTabbedPane();
				tabPaneSuper.addTab(org.getName(), tabpane);
				List<JComponent> components = VisualiseProperties.visualiseProperties(org, editable); 
				for(JComponent comp: components){
					tabpane.add(comp);
				}
			}
		}
		return tabPaneSuper;
	}
}
